#!/usr/bin/env python
##################################################
# File Name: gini.py
# Author: Rui
# mail: rdong1989@gmail.com
# Created Time: Mon 08 Jul 2019 03:40:50 PM EDT
################################################

import numpy as np
import scanpy as sc
import anndata
import math
import umap
from sklearn import preprocessing
from scipy.interpolate import interp1d
import statsmodels.api as sm
from scipy import stats
import warnings
warnings.filterwarnings('ignore')

def giniIndexCalculation(array):
    plusArray=array+0.0000001
    plusArray=np.sort(plusArray)
    index=np.arange(1,plusArray.shape[0]+1)
    n=plusArray.shape[0]
    indexValue=(np.sum((2*index-n-1)*plusArray))/(n*np.sum(plusArray))
    return (indexValue)

def giniIndex(filteredArray):
    funcGeneGini=[]
    funcGeneMax=[]
    for i in range(0,len(filteredArray)):
        exp=filteredArray[i]
        maxExp=np.max(exp)
        giniIndexValue=giniIndexCalculation(exp)
        funcGeneGini.append(giniIndexValue)
        funcGeneMax.append(float(maxExp))
    return(funcGeneGini,funcGeneMax)

def arctanTransform(matrix):
    transMatrix=matrix.T
    binCellNum=int(len(matrix)/1000)
    expMList=[]
    arctanExp=[]
    for i in range(0,len(transMatrix)):
        expSort=np.sort(transMatrix[i])[::-1]
        expSum=np.sum(expSort)
        expM=[]
        if (binCellNum<=9):
            for j in range(0,len(expSort)):
                binSum=np.sum(expSort[0:j])
                div=binSum/expSum
                if div>0.8:
                    expM=np.mean(expSort[0:j])
                    break
        else:
            loopNum=int((len(matrix)-binCellNum)/binCellNum)
            for j in range(loopNum):
                end=(j+1)*binCellNum
                binSum=np.sum(expSort[0:end])
                div=binSum/expSum
                if div>0.8:
                    expM=np.mean(expSort[0:end])
                    break
        arctan=np.arctan((transMatrix[i]-expM))*10+np.arctan(expM)*10
        arctanExp.append(arctan)
    arctanExp=np.array(arctanExp)
    return(arctanExp.T)

def loessRegression(funcGeneGini,funcLogGeneMax,funcGene,funcGiniCutoff,funcPvalue):
    dictGini={}
    for i in range(len(funcGene)):
        dictGini[funcGene[i]]=funcGeneGini[i]
    fit=sm.nonparametric.lowess(funcGeneGini,funcLogGeneMax,frac=0.9)
    f=interp1d(list(zip(*fit))[0],list(zip(*fit))[1], bounds_error=False)
    giniFit=f(funcLogGeneMax)
    residue=funcGeneGini-giniFit
    posRes=[]
    for i in range(len(residue)):
        posRes.append(residue[i])
    residueSort=sorted(posRes)
    quarter=int(len(residueSort)*3/4)
    cutoff=residueSort[quarter]
    quantileGini=[]
    quantileLogMax=[]
    quantileGene=[]
    outlierGini=[]
    outlierLogMax=[]
    outlierGene=[]
    for i in range(0,len(residue)):
        if (residue[i]<=cutoff):
            quantileGene.append(funcGene[i])
            quantileGini.append(funcGeneGini[i])
            quantileLogMax.append(funcLogGeneMax[i])
        else:
            outlierGini.append(funcGeneGini[i])
            outlierGene.append(funcGene[i])
            outlierLogMax.append(funcLogGeneMax[i])
    fit2=sm.nonparametric.lowess(np.array(quantileGini),np.array(quantileLogMax),frac=0.9)
    reFit=interp1d(list(zip(*fit2))[0],list(zip(*fit2))[1], bounds_error=False)
    quantileGiniReFitPredict = reFit(np.array(quantileLogMax))
    dictFunction={}
    for i in range(len(quantileLogMax)):
        dictFunction[quantileLogMax[i]]=quantileGiniReFitPredict[i]
    uniqQuantileLogMax=list(dictFunction.keys())
    sortQuantileLogMax=sorted(uniqQuantileLogMax,reverse=True)
    k=(dictFunction[sortQuantileLogMax[1]]-dictFunction[sortQuantileLogMax[2]])/(sortQuantileLogMax[1]-sortQuantileLogMax[2])
    b=dictFunction[sortQuantileLogMax[1]]-k*sortQuantileLogMax[1]
    outlierGiniReFitPredict = reFit(outlierLogMax)
    ###remove value NaN###
    for i in range(0,len(outlierGiniReFitPredict)):
        if math.isnan(outlierGiniReFitPredict[i]):
            outlierGiniReFitPredict[i]=outlierLogMax[i]*k+b
    reFitResidueQuantile=quantileGini-quantileGiniReFitPredict
    reFitResidueOutlier=outlierGini-outlierGiniReFitPredict
    reFitResidue=np.array(list(reFitResidueQuantile)+list(reFitResidueOutlier))
    newGene=quantileGene+outlierGene
    newGini=np.array(list(quantileGini)+list(outlierGini))
    newFitGini=np.array(list(quantileGiniReFitPredict)+list(outlierGiniReFitPredict))
    newLogMax=quantileLogMax+outlierLogMax
    newResidualGini=newGini-newFitGini
    pvalue=stats.norm.cdf(-abs(preprocessing.scale(newResidualGini)))
    funcSigGiniGeneGini={}
    funcSigGiniGenePvalue={}
    for i in range(0,len(pvalue)):
        if (float(pvalue[i])<funcPvalue and dictGini[newGene[i]]>=funcGiniCutoff):
            funcSigGiniGeneGini[newGene[i]]=dictGini[newGene[i]]
            funcSigGiniGenePvalue[newGene[i]]=pvalue[i]
    return(funcSigGiniGeneGini,funcSigGiniGenePvalue)

def giniValueSelectionM(funcGeneGini,funcGene,funcGiniCutoff):
    funcSigGiniGeneGini={}
    for i in range(0,len(funcGene)):
        if (float(funcGeneGini[i])>=funcGiniCutoff):
            funcSigGiniGeneGini[funcGene[i]]=funcGeneGini[i]
    return(funcSigGiniGeneGini)

def calGini(adataSC,**kwargs):
    """
    Calculate Gini Index value for each gene.
    Params
    ------
    adata: Anndata
        The annotated data matrix of shape `n_obs` × `n_vars`.
        Rows correspond to cells and columns to genes.
    selection: string, optional (Default: 'p_value')
        selection='p_value' or selection='gini_value'. 'p_value' mode indicates
        Gini genes are selected by p value of Loess regression. 'gini_value' mode
        indicates Gini genes are selected by Gini Index value. Recommend 'p_value' 
        mode.
    p_value: float, optional (Default=0.0001)
        If `selection='p_value'`, assign a p value cutoff for Gini gene selection.
    min_gini_value: float, optional (Default=0.6)
        Assign min Gini Index value cutoff in Gini gene selection. The Gini
        Index value is range from 0 to 1. Thus the min_gini_value cutoff is 
        better in range from 0 to 0.8. Larger value indicates a more stringent
        in select Gini genes, smaller values indicates more Gini genes are 
        selected

    A. Gini gene selection in 'p_value' mode: 1. p value < p_value 2. Gini Index
    value >= min_gini_value. (Recommended)
    B. In 'gini_value' mode: Gini Index value > min_gini_value.

    Returns
    -------
    Returns dictionary with Gini genes. adata.var['gini']
    """

    ginicutoff=kwargs.get('min_gini_value', 0.6)
    p_value=kwargs.get('p_value', 0.0001)
    selection=kwargs.get('selection', 'p_value')
    geneLabel=adataSC.var.index.tolist()
    cellLabel=adataSC.obs.index.tolist()
    print("Gene number is "+str(len(geneLabel)))
    print("Cell number is "+str(len(cellLabel)))

    ###calculate gini index###
    geneGini,geneMax=giniIndex(adataSC.X.T)
    allGini=np.array(list(zip(geneLabel,geneGini)))
    logGeneMax=list(map(lambda x:math.log(x+0.1,2),geneMax))
    geneGini=np.array(geneGini)
    geneMax=np.array(geneMax)
    logGeneMax=np.array(logGeneMax)
    sigGiniGene={}
    if (selection=='p_value'):
        sigGiniGene,sigGiniPvalue=loessRegression(geneGini,logGeneMax,geneLabel,ginicutoff,p_value)
    elif (selection=='gini_value'):
         sigGiniGene=giniValueSelectionM(geneGini,geneLabel,ginicutoff)
    else:
        raise SystemExit("Only p_value or gini_value mode is allowed in this step.")
    if (len(sigGiniGene)<2):
        raise SystemExit("Only "+str(len(sigGiniGene))+" Gini gene passed the cutoff, set a lower value of --min_gini_value.")

    ###Select and save high Gini genes to Anndata###
    geneGiniBool=[]
    for i in range(len(geneLabel)):
        value=0
        if geneLabel[i] in sigGiniGene.keys():
            value=1
        geneGiniBool.append(value)
    adataSC.var['gini']=np.array(geneGiniBool,dtype=bool)

def clusterGini(adataSC,**kwargs):
    """
    Cluster cell based on Gini Index value.
    Params
    ------
    adata: Anndata
        The annotated data matrix of shape `n_obs` × `n_vars`.
        Rows correspond to cells and columns to genes.
    neighbors: int, optional (Default=5)
        The size of local neighborhood used for manifold approximation. Larger
        values result in more global views of the manifold, while smaller values
        result in more local data being preserved. For rare cell identification
        this values should be in the range 2 to 15. Recommended neighbors = 5.
    resolution: float, optional (Default=0.1)
        A parameter value controlling the coarseness of the clustering. Higher 
        values lead to more clusters.
    method: string, optional (Default: 'leiden')
        method='louvain' or method='leiden'.

    Returns
    -------
    Returns dictionary with gini cluster result. adata.var['rare']
    """
    
    cluster_neighbors=kwargs.get('neighbors', 5)
    cluster_resolution=kwargs.get('resolution', 0.1)
    cluster_method=kwargs.get('method', "leiden")
    if (cluster_method!="louvain" and cluster_method!="leiden"):
        raise SystemExit("Only leiden or louvain cluster method is allowed in this step.")
    adataGini=adataSC[:,adataSC.var['gini']]
    scaleMatrix=arctanTransform((adataGini.X))
    adataScaleGini=anndata.AnnData(X=scaleMatrix)

    ###calculate neighbor and clustering###
    sc.pp.neighbors(adataScaleGini,use_rep='X',n_neighbors=cluster_neighbors)
    giniClust=[]
    if (cluster_method=="louvain"):
        sc.tl.louvain(adataScaleGini,resolution=cluster_resolution)
        giniClust=adataScaleGini.obs['louvain'].values.tolist()
    else:
        sc.tl.leiden(adataScaleGini,resolution=cluster_resolution)
        giniClust=adataScaleGini.obs['leiden'].values.tolist()
    adataSC.obs['rare']=giniClust
    return(adataScaleGini)



#!/usr/bin/env python
##################################################
# File Name: gini.py
# Author: Rui
# mail: rdong1989@gmail.com
# Created Time: Mon 08 Jul 2019 03:40:50 PM EDT
################################################

import scanpy as sc

def plotGini(adataGini,**kwargs):
    """
    Umap visualization of rare clusters.
    Params
    ------
    adata: Anndata
        The annotated data matrix of shape `n_obs` × `n_vars`.
        Rows correspond to cells and columns to genes.
    method: string, optional (Default: 'umap')
        'method' should be 'umap' or 'tsne'.

    Return
    -------
    UMAP or T-SNE plot in "figure/finalCluster_gini.pdf"
    """

    method=kwargs.get('method','umap')
    if (method=='umap'):
        sc.tl.umap(adataGini)
        sc.pl.umap(adataGini,color='final',save="_finalCluster_umap_gini.pdf")
    elif(method=='tsne'):
        sc.tl.tsne(adataGini)
        sc.pl.tsne(adataGini,color='final',save="_finalCluster_tsne_gini.pdf")
    else:
        raise SystemExit("'method' should be 'umap' or 'tsne'.")   

def plotFano(adataFano,**kwargs):
    """
    Umap visualization of rare clusters.
    Params
    ------
    adata: Anndata
        The annotated data matrix of shape `n_obs` × `n_vars`.
        Rows correspond to cells and columns to genes.
    method: string, optional (Default: 'umap')
        'method' should be 'umap' or 'tsne'.

    Return
    -------
    UMAP or T-SNE plot in "figure/umap_finalCluster_based_on_fano.pdf"
    """
    method=kwargs.get('method','umap')
    if (method=='umap'):
        sc.tl.umap(adataFano)
        sc.pl.umap(adataFano,color='final',save="_finalCluster_umap_fano.pdf")
    elif(method=='tsne'):
        sc.tl.tsne(adataFano)
        sc.pl.tsne(adataFano,color='final',save="_finalCluster_tsne_fano.pdf")
    else:
        raise SystemExit("'method' should be 'umap' or 'tsne'.")



#!/usr/bin/env python
##################################################
# File Name: consensus.py
# Author: Rui
# mail: rdong1989@gmail.com
# Created Time: Thu 11 Jul 2019 03:08:59 PM EDT
################################################

from collections import defaultdict
import anndata
import scanpy as sc
import numpy as np
from sklearn.cluster import KMeans

def calMPG(funcGiniClust,funcMinpts):
    funcGiniDict={}
    for i in range(0,len(funcGiniClust)):
        funcGiniDict[funcGiniClust[i]]=funcGiniDict.get(funcGiniClust[i],0)+1
    s=0.49*(funcMinpts/len(funcGiniClust))
    m=(funcMinpts/len(funcGiniClust))*4
    funcRare=[]
    for keys in funcGiniDict.keys():
        funcGiniDict[keys]=funcGiniDict[keys]/len(funcGiniClust)
        if (funcGiniDict[keys]<m):
            funcRare.append(keys)
        funcGiniDict[keys]=1-1/(1 + np.exp(-(funcGiniDict[keys]-m)/s))
    return(funcGiniDict)

def calMtilde(funcNormGiniClust,funcGiniClust,funcFanoClust,simMtilde):
    for i in range(0,funcGiniClust.shape[0]):
        for j in range(0,funcGiniClust.shape[0]):
            if (i!=j):
                if funcGiniClust[i]==funcGiniClust[j]:
                    MPG=1
                else:
                    MPG=0
                if funcFanoClust[i]==funcFanoClust[j]:
                    MPF=1
                else:
                    MPF=0
                GWeights=max(funcNormGiniClust[i],funcNormGiniClust[j])
                normG=GWeights/(GWeights+0.1)
                simMtilde[i][j]=MPG*normG+MPF*(1-normG)
            else:
                simMtilde[i][j]=1

def determinK(gini,fano):
    cellNum=0
    for g in gini.keys():
        cellNum=cellNum+len(gini[g])
    count=0
    rare=0
    for g in gini.keys():
        if (len(gini[g])/cellNum>0.01):
            continue
        rare=rare+1
        for f in fano.keys():
            overlap=len(np.intersect1d(gini[g],fano[f]))
            overlapG=overlap/len(gini[g])
            overlapF=overlap/len(fano[f])
            if (overlapG>=0.8 and overlapF>=0.8):
                count+=1
    k=rare+len(fano)-count
    return(k)

def overlapGF(gini,fano,giniList):
    giniNoOverlap={}
    for g in gini.keys():
        giniNoOverlap[int(g)]=int(g)
        if (g==0):
            continue
        for f in fano.keys():
            overlap=len(np.intersect1d(gini[g],fano[f]))
            overlapG=overlap/len(gini[g])
            overlapF=overlap/len(fano[f])
            if (overlapG>=0.8 and overlapF>=0.8):
                giniNoOverlap[int(g)]=0
    for i in range(len(giniList)):
        giniList[i]=giniNoOverlap[giniList[i]]
    return(giniList)

def generateMtilde(GCconsensus):
    """
    Generate Mtilde matrix based on Gini and Fano cluster results
    Params
    ------
    GCdict
        GC dict for Gini and Fano cluster results

    Returns
    -------
    Returns dictionary with consensus matrix Mtilde and associated information.
    GCconsensus['Mtilde']=simMtilde
    GCconsensus['overlap']=overlapGiniClust
    GCconsensus['giniCellDict']=giniCellDict
    GCconsensus['fanoCellDict']=fanoCellDict
    GCconsensus['giniIndex']=giniIndex
    GCconsensus['fanoIndex']=fanoIndex
    """
    giniCluster=GCconsensus['giniCluster']
    fanoCluster=GCconsensus['fanoCluster']
    count=1
    giniCellDict = defaultdict(list)
    for i in range(len(giniCluster)):
        giniCellDict[giniCluster[i]].append(count)
        count+=1
    giniCluster=np.array(giniCluster,dtype=int)
    count=1
    fanoCellDict = defaultdict(list)
    for i in range(len(fanoCluster)):
        fanoCellDict[fanoCluster[i]].append(count)
        count+=1
    overlapGiniClust=overlapGF(giniCellDict,fanoCellDict,giniCluster)
    minpts=len(overlapGiniClust)/500
    giniClustDict=calMPG(overlapGiniClust,minpts)
    giniIndex=[]
    fanoIndex=[]
    hashUni={}
    for i in range(len(overlapGiniClust)):
        key=str(overlapGiniClust[i])+"_"+str(fanoCluster[i])
        if key not in hashUni.keys():
            giniIndex.append(overlapGiniClust[i])
            fanoIndex.append(fanoCluster[i])
            hashUni[key]=1
    giniIndex=np.array(giniIndex)
    fanoIndex=np.array(fanoIndex)
    normGiniIndex=[]
    for i in range(len(giniIndex)):
        normGiniIndex.append(giniClustDict[giniIndex[i]])
    normGiniIndex=np.array(normGiniIndex,dtype='float')
    simMtilde=np.zeros((len(giniIndex),len(fanoIndex)),dtype='float64')
    calMtilde(normGiniIndex,giniIndex,fanoIndex,simMtilde)
    GCconsensus['Mtilde']=simMtilde
    GCconsensus['overlap']=overlapGiniClust
    GCconsensus['giniCellDict']=giniCellDict
    GCconsensus['fanoCellDict']=fanoCellDict
    GCconsensus['giniIndex']=giniIndex
    GCconsensus['fanoIndex']=fanoIndex

def clusterMtilde(GCconsensus,**kwargs):
    """
    Cluster consensus Mtilde matrix based on Gini and Fano cluster results.
    Params
    ------
    GCDict
        GCDict returned from generateMtilde
    k: int, optional 
        Number of K in KMeans clustering. Default value based on Gini and Fano
        cluster results.

    Returns
    -------
    Returns dictionary with KMeans cluster result. GCDict['finalCluster']
    """
    k_auto=determinK(GCconsensus['giniCellDict'],GCconsensus['fanoCellDict'])
    K=kwargs.get('k',k_auto)
    kmeans = KMeans(n_clusters=K,n_init=100).fit(GCconsensus['Mtilde'])
    finalClustIndex=kmeans.labels_
    GCconsensus['finalIndex']=np.array(finalClustIndex)

    ###project to each single cell###
    hashProject={}
    for i in range(len(GCconsensus['finalIndex'])):
        key=str(GCconsensus['giniIndex'][i])+"_"+str(GCconsensus['fanoIndex'][i])
        hashProject[key]=GCconsensus['finalIndex'][i]
    finalClust=[]
    for i in range(len(GCconsensus['overlap'])):
        key=str(GCconsensus['overlap'][i])+"_"+str(GCconsensus['fanoCluster'][i])
        finalClust.append(str(hashProject[key]))

    ######sort and remark clusters######
    remarkClust=[]
    hashFinalCount={}
    for i in range(len(finalClust)):
        hashFinalCount[finalClust[i]]=hashFinalCount.get(finalClust[i],0) + 1
    diCount=[]
    for i in hashFinalCount.keys():
        listCount=[]
        listCount.append(i)
        listCount.append(hashFinalCount[i])
        diCount.append(listCount)
    diCount.sort(key=lambda x:x[1],reverse=True)
    hashFinalProject={}
    for i in range(len(diCount)):
        hashFinalProject[diCount[i][0]]=str(i)
    finalSortedClust=[]
    for i in range(len(finalClust)):
        finalSortedClust.append(hashFinalProject[finalClust[i]])
    GCconsensus['finalCluster']=finalSortedClust



#!/usr/bin/env python
##################################################
# File Name: fano.py
# Author: Rui
# mail: rdong1989@gmail.com
# Created Time: Mon 08 Jul 2019 04:08:00 PM EDT
################################################

import numpy as np
import scanpy as sc
import anndata
import warnings
warnings.filterwarnings('ignore')

def calFanoFactor(expArray,geneName):
    fanoDict={}
    exp=np.array(expArray,dtype=float)
    expT=exp.T
    fano=[]
    for i in range(0,len(expT)):
        if (np.mean(expT[i])==0):
            fano.append(0)
        else:
            genefano=np.var(expT[i])/np.mean(expT[i])
            fano.append(genefano)
    for i in range(0,len(geneName)):
        fanoDict[geneName[i]]=fano[i]
    fanoSort=sorted(fanoDict.items(),key=lambda item:item[1],reverse=True)
    highFano={}
    for num in range(0,1000):
        highFano[fanoSort[num][0]]=fanoDict[fanoSort[num][0]]
    return(highFano)

def calFano(adataSC,**kwargs):
    """
    Calculate Fano factor value for each gene.
    Params
    ------
    adata: Anndata
        The annotated data matrix of shape `n_obs` × `n_vars`.
        Rows correspond to cells and columns to genes.
    method: string, optional (Default: 'scanpy')
        method='gini2' or method='scanpy'. 'gini2' mode indicates
        Fano factor is calculated based on var/mean as indicated in GiniClust2.
        'scanpy' mode is hvg selection by using scanpy implemented function. 
        Recommend: 'scanpy' mode.

    Returns
    -------
    Returns dictionary with Gini genes. adata.var['highly_variable']
    """
    cluster_method=kwargs.get('method', "scanpy")
    #####cluster by using high Fano genes######
    sc.pp.log1p(adataSC)
    if (cluster_method=="scanpy"):
        sc.pp.highly_variable_genes(adataSC)
    else:
        sigFanoGene=calFanoFactor(adataSC.X,geneLabel)
        geneFanoBool=[]
        for i in range(len(geneLabel)):
            value=0
            if geneLabel[i] in sigFanoGene.keys():
                value=1
            geneFanoBool.append(value)
        adataSC.var['highly_variable']=np.array(geneFanoBool,dtype=bool)

def clusterFano(adataSC,**kwargs):
    """
    Cluster cells based on Fano factor.
    Params
    ------
    adata: Anndata
        The annotated data matrix of shape `n_obs` × `n_vars`.
        Rows correspond to cells and columns to genes.
    neighbors: int, optional (Default=15)
        The size of local neighborhood used for manifold approximation. Larger
        values result in more global views of the manifold, while smaller values
        result in more local data being preserved. This values should be in the 
        range 2 to 100. Recommended neighbors = 15.
    resolution: float, optional (Default=0.1)
        A parameter value controlling the coarseness of the clustering. Higher
        values lead to more clusters. 
    method: string, optional (Default: 'leiden')
        method='louvain' or method='leiden'.

    Returns
    -------
    Returns dictionary with highly variable genes. adata.var['highly_variable']
    """
    cluster_neighbors=kwargs.get('neighbors', 15)
    cluster_resolution=kwargs.get('resolution', 0.1)
    cluster_method=kwargs.get('method', "leiden")
    if (cluster_method!="louvain" and cluster_method!="leiden"):
        raise SystemExit("Only leiden or louvain cluster method is allowed in this step.")
    adataFano = adataSC[:,adataSC.var['highly_variable']]
    sc.pp.scale(adataFano)
    sc.pp.pca(adataFano)
    sc.pp.neighbors(adataFano,n_neighbors=cluster_neighbors)
    if (cluster_method=="louvain"):
        sc.tl.louvain(adataFano,resolution=cluster_resolution)
        adataSC.obs['fano']=adataFano.obs['louvain']
    else:
        sc.tl.leiden(adataFano,resolution=cluster_resolution)
        adataSC.obs['fano']=adataFano.obs['leiden']
    return(adataFano)



#!/usr/bin/env python
##################################################
# File Name: setup.py
# Author: Rui
# mail: rdong1989@gmail.com
# Created Time: Tue 23 Jul 2019 11:17:18 AM EDT
################################################

#!/usr/bin/env python
from setuptools import setup

setup(
    name='giniclust3',
    version='1.1.0',
    description='Rare cluster identification in single cells',
    url='https://github.com/rdong08/GiniClust3',
    author='Rui Dong',
    author_email='rdong1989@gmail.com',
    install_requires = ['numpy>=1.15.4','scikit-learn>=0.20.2','scipy>=1.2.0','statsmodels>=0.9.0'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='GiniClust3 python',
    packages=['giniclust3'],
)



#!/usr/bin/env python
##################################################
# File Name: __init__.py
# Author: Rui
# mail: rdong1989@gmail.com
# Created Time: Tue 23 Jul 2019 12:36:49 PM EDT
################################################

import sys

__author__ = ("Rui Dong")
__email__ = ("rdong1989@gmail.com")

from . import gini
from . import fano
from . import consensus
from . import plot
